/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCadView.h"
#include "registry.h"
#include "TinyCad.h"
#include "TinyCadSymbolDoc.h"


////// The Library Member Functions //////


// Create an empty library unattached to any file
CLibraryFile::CLibraryFile()
{
  // Default the Last Got symbol in this library
  LastGet.name="";
  LastGet.reference="U?";
  LastGet.ppp=1;
  LastGet.lib = this;

  revision = 0;
  methodsFileOpen=FALSE;

  // Default the name
  name="Library";
  next=NULL;
}



// The destructor
CLibraryFile::~CLibraryFile()
{
  // Ensure the last get symbol pointer is cleared before it is deleted!
  if (methodsFileOpen)
	theMethodsFile.Close();
}



// Find a string in the symbol names
void CLibraryFile::Find(const char *theString,CListBox *theListBox)
{
	symbolCollection::iterator it = Symbols.begin();

	while (it != Symbols.end())
	{
		CLibraryFileSymbol *pSymbol = &(it->second);
		CString name = pSymbol->name;
		CString desc = pSymbol->description;
		name.MakeLower();
		desc.MakeLower();
		if (	name.Find(theString) != -1
			||	desc.Find(theString) != -1)
		{
			theListBox->AddString( pSymbol->name + " - " + pSymbol->description );
		}
		++ it;
	}
}



// Create a library associated with an index and methods file
void CLibraryFile::Attach(CString FileName)
{
  CFile theIndex;
  CFileException ep;
  BYTE typ;

  name=FileName;

  if (!OpenIndexFile(theIndex,CFile::modeRead))
	return;
	
  // Open the archive header
  CArchive theIndexArchive(&theIndex,CArchive::load);

  TRY
  {
	// Check the header on the index file
	if (CheckHeaderStamp(theIndexArchive))
		// Read in the symbol table names
		for (;;) {
			theIndexArchive >> typ;
			// Is it the index information part?
			if (typ==xIndexInformation) {
				theIndexArchive >> revision;
			// Is it a symbol definition?
			} else if (typ==xSymbol || typ == xSymbolEx || typ == xSymbolEx2) {
				CLibraryFileSymbol nwSymbol(this);
				// Check for old type of symbol descriptor
				if (typ == xSymbol)
					nwSymbol.OldLoad2(theIndexArchive);
				else if (typ == xSymbolEx)
					nwSymbol.OldLoad1(theIndexArchive);
				else
					nwSymbol.Load(theIndexArchive);
					
				Symbols[nwSymbol.name] = nwSymbol;
			// No, must be a corrupted file!
			} else {
				Message(IDS_BADLIB,MB_ICONEXCLAMATION | MB_OK,name);
				theIndexArchive.Close();
				methodsFileOpen=FALSE;
				return;
			}
		}
  }
  CATCH( CArchiveException, e )
  {
	// end of files are allowed, all other exceptions are not
	if (e->m_cause!=CArchiveException::endOfFile) {
		Message(IDS_BADLIB,MB_ICONEXCLAMATION | MB_OK,FileName);
	}
  }
  AND_CATCH( CException, e)
  {
	Message(IDS_BADLIB,MB_ICONEXCLAMATION | MB_OK,FileName);
  }
  END_CATCH

  theIndexArchive.Close();

  methodsFileOpen=FALSE;
}



// Re-read the library
void CLibraryFile::ReRead()
{
  // Get rid of the hash table
	Symbols.erase( Symbols.begin(), Symbols.end() );

  // Re-read the symbol table in
  Attach(name);
}




// Generate the methods file name from library name and revision number
CString CLibraryFile::MethodsFileName()
{
  CString hold = name + ".m";
  char buffer[10];

  _itoa(revision,buffer,10);

  if (strlen(buffer)<2)
	hold = hold + "0";
  hold = hold + buffer;

  return hold;
}

// Get the methods file associated with this library
CFile *CLibraryFile::GetMethodFile()
{
  CFileException ep;

  if (!methodsFileOpen) {
	  if (!theMethodsFile.Open(MethodsFileName(),CFile::modeRead,&ep)) {
		Message(IDS_MISSLIB,MB_ICONEXCLAMATION | MB_OK,name);
		methodsFileOpen = FALSE;
		return NULL;
	}
	methodsFileOpen=TRUE;
  }

  return &theMethodsFile;
}



// Close the shared methods file, if it is currently open
void CLibraryFile::CloseMethodFile()
{
  if (methodsFileOpen) {
	theMethodsFile.Close();
	methodsFileOpen=FALSE;
  }
}




// Open the index file
BOOL CLibraryFile::OpenIndexFile(CFile &theIndexFile,int mode)
{
  CFileException ep;

  if (!theIndexFile.Open(name+".idx",mode,&ep)) {
	Message(IDS_MISSLIB,MB_ICONEXCLAMATION | MB_OK,name);
	return FALSE;
  }

  return TRUE;
}




// Extract a symbol from this library
CLibraryFileSymbol *CLibraryFile::Extract(const char *key)
{
  CLibraryFileSymbol *temp=NULL;
  
  if (Symbols.find(key) != Symbols.end())
  {
	  temp = &(Symbols[key]);
  }

  if (temp!=NULL)
	LastGet=*temp;

  return temp;
}



// Tidy up the library
void CLibraryFile::Tidy()
{
  // Show the busy icon
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );

  // Build a linked list of the items in this library
  symbolCollection temp_symbols;

  CFile theIndex;
  CFileException ep;
  BYTE typ;

  if (!OpenIndexFile(theIndex,CFile::modeRead))
	return;
	
  // Open the archive header
  CArchive theIndexArchive(&theIndex,CArchive::load);

  try
  {
	// Check the header on the index file
	if (CheckHeaderStamp(theIndexArchive))
	{

		// Read in the symbol table names
		for (;;) 
		{
			theIndexArchive >> typ;
			// Is it the index information part?
			if (typ == xIndexInformation) 
			{
				theIndexArchive >> revision;
			} 
			// Is it a symbol definition?
			else if (typ==xSymbol || typ == xSymbolEx || typ == xSymbolEx2) 
			{
				// Add the symbol to the linked list
				CLibraryFileSymbol nwSymbol(this);

				// Check for old type of symbol descriptor
				if (typ == xSymbol)
				{
					nwSymbol.OldLoad2(theIndexArchive);
				}
				else if (typ == xSymbolEx)
				{
					nwSymbol.OldLoad1(theIndexArchive);
				}
				else
				{
					nwSymbol.Load(theIndexArchive);
				}
				
				// Has this symbol name been used before?
				symbolCollection::iterator it = temp_symbols.find( nwSymbol.name );
				temp_symbols[ nwSymbol.name ] = nwSymbol;
			} 
			else 
			{
				CDialog theDialog("BadSymbol");
				theDialog.DoModal();
				theIndexArchive.Close();
				methodsFileOpen=FALSE;
				return;
			}
		}
	}
  }
  catch( CArchiveException *e )
  {
	// end of files are allowed, all other exceptions are not
	if (e->m_cause != CArchiveException::endOfFile) 
	{
		Message(IDS_BADLIB,MB_ICONEXCLAMATION | MB_OK,name);
	}
	e->Delete();
  }
  catch( CException *e)
  {
	Message(IDS_BADLIB,MB_ICONEXCLAMATION | MB_OK,name);
	e->Delete();
  }

  theIndexArchive.Close();
  theIndex.Close();

  SaveSymbolCollection( temp_symbols );

  Message(IDS_LIBTIDIED);
}


void CLibraryFile::SaveSymbolCollection( symbolCollection &temp_symbols )
{
  // Now write back the information contained in the index file, and
  // use it to delete unecessary information in the methods file

  // Open the old methods file for reading
  CFile theIndex;
  CFile OldMethods,NewMethods;
  CString OldMethodsName = MethodsFileName();
  int ok = OldMethods.Open(OldMethodsName,CFile::modeRead);

  // change the revision number
  revision++;
  if (revision>99)
	revision = 0;

  // Open a new methods file for writing to
  ok &= NewMethods.Open(MethodsFileName(),CFile::modeCreate | CFile::modeWrite);

  // Now re-open the index file
  if (!OpenIndexFile(theIndex,CFile::modeCreate | CFile::modeWrite) | !ok) {
	Message(IDS_MISSLIB,MB_ICONEXCLAMATION | MB_OK,name);
	return;
  }

  CArchive theIndexSave(&theIndex,CArchive::store);
  HeaderStamp(theIndexSave);

  // Now write the library revision information
  theIndexSave << (BYTE)xIndexInformation;
  theIndexSave << revision;

  // Now we can write the new index file back
  DWORD OldPos=-1,NewPos=-1;

  CTinyCadDoc tmp_design;

	symbolCollection::iterator it = temp_symbols.begin();
  while (it != temp_symbols.end()) 
  {
	CLibraryFileSymbol thisSymbol = it->second;

	// Move the old method from the old file to the new file
	OldPos = thisSymbol.FilePos;
	NewPos = NewMethods.GetPosition();
	
	// Read the methods file into this design
	OldMethods.Seek( OldPos, CFile::begin);
	CArchive ar( &OldMethods, CArchive::load );
	tmp_design.Load(ar);

	// Now save this design onto the end of the new methods file
	CArchive ar2( &NewMethods, CArchive::store );
	tmp_design.Save(ar2,FALSE,TRUE);

	// Now remove this design from memory
	tmp_design.SelectDelete();

	// Now we can write this symbol into the new index file
	thisSymbol.FilePos = NewPos;
	theIndexSave << (BYTE)xSymbolEx2;
	thisSymbol.Save(theIndexSave);

	// Next symbol
	it ++;
  }  


  // Close all the files to ensure the data is written correctly
  theIndexSave.Close();
  theIndex.Close();
  NewMethods.Close();
  OldMethods.Close();


  // Now re-read this library
  ReRead();
  
  // Delete the old methods file
  CFile::Remove(OldMethodsName);

  // Now inform the user it has been done
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );
}


// Write a symbol to this library
void CLibraryFile::Store( CTinyCadSymbolDoc &document )
{
	BOOL ok;

	// Set the busy cursor
	SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );

	// Close down the current access to the file
	CloseMethodFile();

	// Now save the symbol into a file

	// Open the file, and move to the end of it
	CFile theMethods,theIndex;
	CFileException e;

	ok = theMethods.Open(MethodsFileName(),CFile::modeReadWrite,&e)
		&& theIndex.Open(name+".idx",CFile::modeReadWrite,&e);

	while (!ok) 
	{
		// Does this file exist?
		if (e.m_cause==CFileException::fileNotFound) 
		{
			revision = 0;

			if (Message(IDS_NOLIBRARY,MB_YESNO | MB_ICONQUESTION)!=IDYES)
			{
				return;
			}

			ok = theMethods.Open(MethodsFileName(),CFile::modeCreate | CFile::modeReadWrite)
			  && OpenIndexFile(theIndex,CFile::modeCreate | CFile::modeReadWrite);

			// Write the header to the Index file
			if (ok) 
			{
				CArchive theIndexArchive(&theIndex,CArchive::store);
				HeaderStamp(theIndexArchive);
				// Now write the library revision information
				theIndexArchive << (BYTE)xIndexInformation;
				theIndexArchive << revision;
			}
		}
		else
		{
			SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );
			Message(IDS_BADLIB,MB_ICONEXCLAMATION | MB_OK,name);
			return;
		}
	}


	// Write the symbol data into the methods file
	theMethods.SeekToEnd();
	document.getSymbol()->FilePos=theMethods.GetPosition();
	{
		CArchive ar( &theMethods, CArchive::store );
		document.Save(ar, FALSE, FALSE);
	}

	// Write the index data into the index file

	// check the header stamp in the index file
	theIndex.SeekToBegin();
	CArchive theIndexArchiveIn(&theIndex,CArchive::load);
	if (!CheckHeaderStamp(theIndexArchiveIn)) 
	{
		SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );
		Message(IDS_BADLIB,MB_ICONEXCLAMATION | MB_OK,name);
		theIndexArchiveIn.Close();
		return;
	}
	theIndexArchiveIn.Close();

	// now write the index entry to the end of the file
	theIndex.SeekToEnd();
	CArchive theIndexArchive(&theIndex,CArchive::store);
	theIndexArchive << (BYTE)xSymbolEx2;
	document.getSymbol()->Save(theIndexArchive);
	theIndexArchive.Close();

	// Close the files
	theIndex.Close();
	theMethods.Close();

	// Inform the design it has been saved
	document.SetModifiedFlag(FALSE);

	// Re-load the library now it has changed
	ReRead();

	// Inform the document views
	static_cast<CTinyCadApp*>(AfxGetApp())->ResetAllSymbols();

	SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );
}
