/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#ifndef __OBJECT_H__
#define __OBJECT_H__


// The different types of object
enum ObjType {
	xEndFile = 0, xNoDraw, xAnotation, xDesignInformation, xIndexInformation, xSymbol,
	xError, xFont, xSymbolEx, xLayerOld, xLineStyle, xFillStyle, xLayer, xLayerTable, 
	xSymbolEx2,

	xBusName = 15,
	xNoConnect = 16, xJunction, xLabel, xWire, xBus, xMethod,
	xPower, xPin, xBusSlash, xLabelEx, xMethodEx, xMethodEx2, xPinEx,

	xLine = 32, xDash, xText, xCircle, xSquare, xArc, xLineEx, xTextEx,
	xCircleEx, xSquareEx, xArcEx,

	xLineEx2, xCircleEx2, xSquareEx2, xArcEx2, xMetaFile,

	xTag  = 48,

	xEditItem = 128, 
	xDesignInformation2,
	xMetaFiles,
	xOptions,
	xSymbols
};


enum CanDo {
	cdoLineWidth  = 1,
	cdoFontStyle  = 2,
	cdoLayer	  = 4
};

#define cdoDefault  (cdoLineWidth | cdoFontStyle)


inline Max(int a,int b) { return a>b ? a : b; }
inline Min(int a,int b) { return a>b ? b : a; }

// This class is defined in library.h
class CDesignFileSymbol;

class CTinyCadDoc;


//////// The is the main object that all objects inherit from //////

class CDrawingObject {

public:

	CTinyCadDoc	*m_pDesign;					// Our current design that this object belongs to

	CPoint a;								// position of this object (in internal units)
	CPoint b;								// the size of the object, these two points define
											// the bounding box of the object
	CDrawingObject *next;							// The next object in this linked list

	char segment;							// Mode of current edit
	int	 Undo_Level;						// The timestamp when this object was created
	int	 Delete_Level;

	BOOL	DoesExist();		// Has this object been undone or deleted?

	CDrawingObject( CTinyCadDoc *pDesign );
	virtual ~CDrawingObject() { }			// The destructor


	CPoint 	ReadPoint(CArchive &);
	void 	WritePoint(CArchive &, CPoint);


	virtual void Save(CArchive &);			// Save data into file
	virtual void Load(CArchive &);			// Load data into this object

	virtual int IsInside(int,int,int,int);	// Is this object inside this rectangle?
	virtual BOOL CanEdit();					// Is this object editable after placed?
	virtual CString GetName();				// Get the string name of this object
	virtual ObjType GetType();				// Get this object's type

	virtual void TagResources();			// Tag the resources this object is currently using
	virtual int  CanUseOptions();			// Find out which options are appropriate for this object
	virtual CString Find(const char *);		// Find a string in this object
	virtual	void NewOptions();		// Indicate to an object being edited that the options have changed

	virtual void BeginEdit();				// Called before starting editing this object
	virtual void EndEdit();		// Called before deleting this object from current edit

	virtual void Display( BOOL erase = TRUE );
	virtual void Paint(Context &, paint_options  );// Display this object on the screen
	
	virtual void Move(CPoint);	// Follow a mouse movement
	virtual void Place(CPoint,CPoint);	// The user has clicked with the left hand button
	virtual BOOL End();			// The user has clicked with the right hand button
	virtual void EndPlace(CPoint);// The user has released the left hand button
	virtual void AfterEnd(CPoint);// The user has released the right hand button

	// Move fields of this object about
	virtual int IsInsideField(CPoint p);
	virtual void MoveField(int w, CPoint r);
	virtual int SetCursor( CPoint p );
	virtual int SetCursorEdit( CPoint p );


	virtual void Store();					// Store current object
	virtual void Rotate(CPoint,int);		// Rotate this item about a point

	virtual int getMenuID() { return -1; }
};



class CDrawRectOutline : public CDrawingObject
{
protected:

	// The constructor
	CDrawRectOutline( CTinyCadDoc *pDesign )
	: CDrawingObject( pDesign )
	{
	}


	// Redraw this object (including the handles)
	virtual void Display( BOOL erase = TRUE );
	void DrawHandles( Context &dc );

	// Move fields of this object about
	virtual int IsInsideField(CPoint p);
	virtual void MoveField(int w, CPoint r);
	virtual int SetCursorEdit( CPoint p );
};

////// These are the schematic objects //////

class CDrawLine : public CDrawingObject {
	ObjType xtype;

	WORD Style;

	void ToAngle();
  	int rIsInside(CPoint,CPoint,int,int,int,int);
public:

	void Save(CArchive &);
	void Load(CArchive &);
	int IsInside(int,int,int,int);
	ObjType GetType();
	void Paint(Context &dc,paint_options options);
	void Display( BOOL erase = TRUE );
	void Place(CPoint,CPoint);
	BOOL End();
	void Move(CPoint);
	void Store();
	void BeginEdit();
 	void EndEdit();
	void NewOptions();
	CString GetName();
	void TagResources();
	int  CanUseOptions() { return xtype ==xLineEx2 ? cdoLineWidth : 0; };
	virtual int getMenuID();


	// Move fields of this object about
	virtual int IsInsideField(CPoint p);
	virtual void MoveField(int w, CPoint r);
	virtual int SetCursorEdit( CPoint p );
	void DrawHandles( Context &dc );


	// This is used for the construction of this object
	CDrawLine(CTinyCadDoc *pDesign, ObjType NewType = xLineEx2);
};



class CDrawJunction : public CDrawingObject {
public:
	int IsInside(int,int,int,int);
	void Display( BOOL erase = TRUE );
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	ObjType GetType();		// Get this object's type
	CString GetName();
	virtual int getMenuID() { return IDM_TOOLJUNC; }

	// This is used for the construction of this object
	CDrawJunction(CTinyCadDoc *pDesign);
};

class CDrawBusSlash : public CDrawingObject {
	BYTE theDir;
public:
	int IsInside(int,int,int,int);
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	ObjType GetType();		// Get this object's type
	CString GetName();
	virtual int getMenuID();
	void Display( BOOL erase = TRUE );

	// These are used for the construction of this object
	CDrawBusSlash(CTinyCadDoc *pDesign,int NewDir = 0);
};

class CDrawNoConnect : public CDrawingObject {
public:
	int IsInside(int,int,int,int);
	void Display( BOOL erase = TRUE );
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	ObjType GetType();		// Get this object's type
	CString GetName();
	virtual int getMenuID() { return IDM_TOOLCONNECT; }

	// This is used for the construction of this object
	CDrawNoConnect(CTinyCadDoc *pDesign);
	virtual ~CDrawNoConnect() { }
};


class CEditDlgTextEdit;

class CDrawText : public CDrawRectOutline
{

	friend CEditDlgTextEdit;

	// The font style
	hFONT	FontStyle;

	CString str;
	BYTE dir;
	ObjType xtype;
public:
	void TagResources();
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	void BeginEdit();
	void EndEdit();
	ObjType GetType();			// Get this object's type
	void Rotate(CPoint,int);
	BOOL CanEdit();
	int  CanUseOptions() { return cdoFontStyle; };


	CString Find(const char *);	// Does this string match this text?
	CString GetName();
	CString GetValue() { return str; }
	void NewFont(LOGFONT *);	// Change the font
	void NewOptions();

	virtual int getMenuID();
	virtual void MoveField(int w, CPoint r);

	// These are used for the construction of this object
	CDrawText(CTinyCadDoc *pDesign,ObjType NewType = xLabelEx);
	virtual ~CDrawText() { }
};


class CEditDlgBusNameEdit;

class CDrawBusName : public CDrawingObject {

	friend CEditDlgBusNameEdit;

	// The font style
	hFONT	FontStyle;

	CString str;
	BYTE dir;

public:
	void TagResources();

	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	void BeginEdit();
	void EndEdit();
	ObjType GetType();			// Get this object's type
	void Rotate(CPoint,int);
	BOOL CanEdit();
	void Display( BOOL erase = TRUE );

	CString Find(const char *);	// Does this string match this text?
	CString GetName();
	CString GetValue() { return str; }
	void NewFont(LOGFONT *);	// Change the font
	void NewOptions();
	int  CanUseOptions() { return cdoFontStyle; };

	virtual int getMenuID() { return IDM_TOOLBUSNAME; }

	// These are used for the construction of this object
	CDrawBusName(CTinyCadDoc *pDesign);
	virtual ~CDrawBusName() { }
};


class CEditDlgPowerEdit;

class CDrawPower : public CDrawingObject {

	friend CEditDlgPowerEdit;

	CPoint	TextPos; // The position of the text item
	CPoint	TextEnd;

	CString str;
	BYTE dir;	// Direction this item is facing (0=up,1=down,2=left,3=right)
	BYTE which;	// Which type of power item it is
public:
	int IsInside(int,int,int,int);
	void TagResources();
	void Display( BOOL erase = TRUE );
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	void BeginEdit();
	void EndEdit();
	void Rotate(CPoint,int);
	BOOL CanEdit();
	ObjType GetType();		// Get this object's type
	CString GetName();
	CString GetValue() { return str; }
	CString Find(const char *);

	virtual int getMenuID() { return IDM_TOOLPOWER; }

	// These are used for the construction of this object
	CDrawPower( CTinyCadDoc *pDesign );
	virtual ~CDrawPower() { }
};


class CEditDlgPinEdit;
class CDrawMethod;

class CDrawPin : public CDrawingObject {

	friend CEditDlgPinEdit;	// The friendly dialog box to edit this pin

	CString str;	// The pin name
	CString number;	// The pin number
	BYTE show;	// Show name or number?
	BYTE dir;	// Direction this item is facing (0=up,1=down,2=left,3=right)
	BYTE which;	// Which type of power item it is
	BYTE elec;	// Which electrical properties this pin has
	BYTE part;	// Which subpart the item is in
	WORD length;

	void DetermineLayout( Context &dc, CPoint &pa,CPoint &pb,CPoint &pc,CPoint &pd,CPoint &pta,CPoint &ptb, int &dr );

public:
	int IsInvisible();	// Is this pin currently visible?
			   		// Is this pin a power pin?
	int IsPower() { return which == 4; }

	int IsInside(int,int,int,int);
	void TagResources();
	void Display( BOOL erase = TRUE );
	void Paint(Context &,paint_options);
	void Store();
	void OldLoad(CArchive &);
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	void BeginEdit();
	void EndEdit();
	BOOL CanEdit();
	void Rotate(CPoint,int);
	ObjType GetType();
	CString GetName();
	CString GetNumber()  { return number; }
	CString GetPinName() { return str; }
	CString Find(const char *);
	int     GetPart()    { return part; }
	int		GetElec()    { return elec; }

	virtual int getMenuID() { return IDM_LIBPIN; }

	// Get the actual position of this pin given the parent method
	CPoint GetPosition(CDrawMethod *parent);

	void SetPart(int NewPart) { part=NewPart; }

	CDrawPin(CTinyCadDoc *pDesign);				// The constructoe
	virtual ~CDrawPin() { }
};

////// An imported advanced metafile //////

class CDrawMetaFile : public CDrawRectOutline
{
	hMETAFILE m_metafile;

public:
	ObjType GetType();
	void TagResources();
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	BOOL End();
	CString GetName();
	void setMetaFile( HENHMETAFILE data );

	// This is used for the construction of this object
	CDrawMetaFile(CTinyCadDoc *pDesign);
	virtual ~CDrawMetaFile();

	virtual int getMenuID() { return IDM_TOOLSQUARE; }
};

////// These are the anotation objects //////

class CDrawSquare : public CDrawRectOutline
{
	WORD Style;

public:
	ObjType GetType();
	void TagResources();
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void OldLoad(CArchive &,int);
	void NewOptions();
	int  CanUseOptions() { return cdoLineWidth; };
	BOOL End();
	CString GetName();

	// This is used for the construction of this object
	CDrawSquare(CTinyCadDoc *pDesign);

	virtual int getMenuID() { return IDM_TOOLSQUARE; }
};


class CDrawArcSegment : public CDrawRectOutline
{
	WORD Style;
public:
	ObjType GetType();
	void TagResources();
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void OldLoad(CArchive &,int);
	void NewOptions();
	int  CanUseOptions() { return cdoLineWidth; };
	BOOL End();
	CString GetName();

	virtual int getMenuID() { return IDM_TOOLARC; }

	// This is used for the construction of this object
	CDrawArcSegment(CTinyCadDoc *pDesign);
};


class CDrawCircle : public CDrawRectOutline 
{
	WORD Style;

public:
	ObjType GetType();
	void TagResources();
	void Paint(Context &,paint_options);
	void Store();
	void Load(CArchive &);
	void Save(CArchive &);
	void OldLoad(CArchive &,int);
	void NewOptions();
	int  CanUseOptions() { return cdoLineWidth; };
	BOOL End();
	CString GetName();

	// This is used for the construction of this object
	CDrawCircle(CTinyCadDoc *pDesign);

	virtual int getMenuID() { return IDM_TOOLCIRCLE; }
};



class CDrawError : public CDrawingObject {
	int ErrorNumber;
public:
	ObjType GetType();
	void Paint(Context &,paint_options);
	CString GetName();
	void BeginEdit();
	void EndEdit();
	BOOL CanEdit();
	int IsInside(int,int,int,int);
	int GetErrorNumber() { return ErrorNumber; }
	void SetErrorNumber(int NewE) { ErrorNumber = NewE; }
	void Store();

	// This is used for the construction of this object
	CDrawError(CTinyCadDoc *pDesign,CPoint NewA = CPoint(0,0),int q = -1);

};



// The Tag class

class CDrawTag : public CDrawingObject {
	CString 	TagName;

public:
	ObjType 	GetType() 		{ return xTag; }
	int 		IsInside(int,int,int,int) { return FALSE; }
	void 		Load(CArchive &);
	void 		Save(CArchive &);

	CString		GetTagName() 	{ return TagName; }
	void		SetTagName(const char *NewTagName) { TagName = NewTagName; }

	// This is used for the construction of this object
	CDrawTag(CTinyCadDoc *pDesign, CPoint NewA,const char *NewTagName)
		: CDrawingObject(pDesign) 
	{ 
		a = NewA; 
		TagName = NewTagName; 
	};

	CDrawTag(CTinyCadDoc *pDesign)
		: CDrawingObject(pDesign)
	{ 
		a=CPoint(0,0); 
		TagName = ""; 
	}
	virtual ~CDrawTag() { }
};



////// These are the block objects //////

class CDrawBlockMove : public CDrawingObject {
	int placed;
	CPoint OldPos;
public:
	void Paint(Context &,paint_options);
	CDrawBlockMove(CTinyCadDoc *);
	void EndEdit();
	void Place(CPoint,CPoint);
	BOOL End();
	void Move(CPoint);

	virtual int getMenuID() { return IDM_EDITMOVE; }
};



class CDrawBlockRotate : public CDrawingObject {
	int placed;

public:
	void Paint(Context &,paint_options);
	CDrawBlockRotate(CTinyCadDoc *);
	void EndEdit();
	void BeginEdit();
	void Place(CPoint,CPoint);
	BOOL End();
	void Move(CPoint);
	void ChangeDir(int);

	virtual int getMenuID() { return IDM_EDITROTATE; }
};


class CDrawBlockDup : public CDrawingObject {
	int placed;

public:
	void Paint(Context &,paint_options);
	CDrawBlockDup(CTinyCadDoc *);
	void EndEdit();
	void Place(CPoint,CPoint);
	BOOL End();
	void Move(CPoint);

	virtual int getMenuID() { return IDM_EDITDUP; }
};



class IntList {
public:
	CDrawingObject  *obj;
	BOOL 	MoveA;
	BOOL	MoveB;
	IntList *next;

	IntList(CDrawingObject *NewObj,BOOL NMA, BOOL NMB) { obj=NewObj; MoveA = NMA; MoveB = NMB; next=NULL; }
	virtual ~IntList() { if (next!=NULL) delete next; }
};
class CDrawBlockDrag : public CDrawingObject {
	int placed;
	IntList *head;
	CPoint OldPos;
public:
	void Paint(Context &,paint_options);
	CDrawBlockDrag(CTinyCadDoc *pDesign);
	void EndEdit();
	void Place(CPoint,CPoint);
	BOOL End();
	void Move(CPoint);

	virtual int getMenuID() { return IDM_EDITDRAG; }
};



class CDrawBlockImport : public CDrawingObject {
public:
	CDrawBlockImport(CTinyCadDoc *pDesign);
	void EndEdit();
	void Place(CPoint,CPoint);
	BOOL End();
	void Move(CPoint);

	virtual int getMenuID() { return IDM_FILEIMPORT; }
};




class CDrawBlockExport : public CDrawingObject {
public:
	void Paint(Context &,paint_options);
	CDrawBlockExport(CTinyCadDoc *pDesign);
	void EndEdit();
	void Place(CPoint,CPoint);
	BOOL End();
	void Move(CPoint);

	virtual int getMenuID() { return IDM_FILEEXPORT; }
};




////// These are the single object editing objects //////


class CDrawEditItem : public CDrawingObject {
	BOOL	InMove;
	BOOL	InSelectByDrag;
	int		EditMethodText;
	CPoint  LastPos;

public:
	CDrawEditItem( CTinyCadDoc *pDesign );
	void NewOptions();
	void EndEdit();
	void Place(CPoint,CPoint);
	void EndPlace(CPoint);
	void Move(CPoint);
	void Paint(Context &,paint_options);
	void ReleaseSelection();
	BOOL End();
	void AfterEnd(CPoint);
	int  CanUseOptions();
	ObjType GetType() { return xEditItem; }
	virtual int SetCursor( CPoint p );

	virtual int getMenuID() { return IDM_EDITEDIT; }
};



////// This is the method CDrawingObject for use with library symbols //////

class CEditDlgMethodEdit;

class CDrawMethod : public CDrawingObject {

	friend CEditDlgMethodEdit;		// Both these objects aid the edit of this object
	friend CDrawEditItem;

	// Where this symbol data comes from
	hSYMBOL			m_Symbol;
	CString	SymName;		// The name of this symbol

	BYTE part;				// Which part in the package this is
	BYTE rotate;			// rotation and mirror

	CString Field[4];		// The fields associated with this symbol
	BOOL	Show[3];		// Whether each field should be shown
	CPoint	Pos[3];			// The position of each field

	void NewRotation();		// Calculate new co-ords after a rotation

	void GetSymbolByName();

public:
	void TagResources();

	enum FieldPos { Name=0, Ref, Other, Attrib };

	CPoint GetTr();
	CDesignFileSymbol *GetSymbolData();
	CDrawingObject* ExtractSymbol(CPoint &tr);
	int IsInside(int,int,int,int);

	virtual int IsInsideField(CPoint);		// Is a point pointing at one of our fields?
	virtual void MoveField(int,CPoint);	// Move a field

	void Display( BOOL erase = TRUE );
	void Paint(Context &,paint_options);
	void Store();
	void OldLoad(CArchive &);
	void OldLoad2(CArchive &);
	void Load(CArchive &);
	void Save(CArchive &);
	void Move(CPoint);
	void Place(CPoint,CPoint);
	void BeginEdit();
	void EndEdit();
	BOOL CanEdit();
	void Rotate(CPoint,int);
	BOOL IsNoSymbol();

	ObjType GetType();		// Get this object's type
	CString GetName();

	CString GetRef() { return Field[Ref]; }
	CString GetField(int);		// Get the string value of a field
	CString Find(const char *);
	CPoint  GetFieldPos(int q) { return CPoint(a.x+Pos[q].x,a.y+Pos[q].y); }

	int  GetRefVal();
	int  GetSubPart() { return part; }
	int  GetRotate() { return rotate; }
	void SetRefVal(int value);
	void SetRef(const char *NewRef) { Field[Ref] = NewRef; }
	void SetPart(int NewPart) { part = NewPart; }

	CDrawMethod(CTinyCadDoc *pDesign,hSYMBOL);		// The constructors
	CDrawMethod(CTinyCadDoc *pDesign);
	virtual ~CDrawMethod() 
	{
	}

	virtual int getMenuID() { return IDM_TOOLGET; }
};


////// These are non-displayable objects! //////

// The zoom in/out object
class CDrawCentre : public CDrawingObject {
public:
	CDrawCentre( CTinyCadDoc *pDesign )
		: CDrawingObject( pDesign )
	{
	}

	void Display( BOOL erase );
	ObjType GetType();
	void Place(CPoint,CPoint);		// Zoom In
	BOOL End();				// Zoom Out

	virtual int SetCursor( CPoint p ) { return 9; }
	virtual int getMenuID() { return IDM_VIEWCENTRE; }
};


#endif
