/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCadView.h"
#include "resource.h"
#include "revision.h"
#include "colour.h"
#include "option.h"
#include "registry.h"
#include "layer.h"
#include "diag.h"
#include "symbol.h"



////// The option catorgories //////


Option::Option()
{
	PinLength = 20;
}


Option::Init( CTinyCadDoc *pDesign )
{
	// Keep a track of the design
	m_pDesign = pDesign;

	// Create the default fonts
	LOGFONT theTextFont;
	
	// Initalise the font settings for normal text
 	theTextFont.lfHeight = sTEXT;
  	theTextFont.lfWidth = 0;	
  	theTextFont.lfEscapement = 0;
  	theTextFont.lfOrientation = 0;
  	theTextFont.lfWeight = 400;
  	theTextFont.lfItalic = FALSE;
  	theTextFont.lfUnderline = FALSE;
  	theTextFont.lfStrikeOut = FALSE;
  	theTextFont.lfCharSet = ANSI_CHARSET;
  	theTextFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
  	theTextFont.lfClipPrecision = CLIP_DEFAULT_PRECIS | CLIP_LH_ANGLES;
  	theTextFont.lfQuality = DEFAULT_QUALITY;
  	theTextFont.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
  	strcpy(theTextFont.lfFaceName,fDEFAULT_FONT);
	theFontList = new ListOfFonts(theTextFont);


	// Initalise the font settings for pin text
 	theTextFont.lfHeight = sPIN;
  	theTextFont.lfWidth = 0;	
  	theTextFont.lfEscapement = 0;
  	theTextFont.lfOrientation = 0;
  	theTextFont.lfWeight = 400;
  	theTextFont.lfItalic = FALSE;
  	theTextFont.lfUnderline = FALSE;
  	theTextFont.lfStrikeOut = FALSE;
  	theTextFont.lfCharSet = ANSI_CHARSET;
  	theTextFont.lfOutPrecision = OUT_DEFAULT_PRECIS;
  	theTextFont.lfClipPrecision = CLIP_DEFAULT_PRECIS  | CLIP_LH_ANGLES;
  	theTextFont.lfQuality = DEFAULT_QUALITY;
  	theTextFont.lfPitchAndFamily = DEFAULT_PITCH | FF_SWISS;
  	strcpy(theTextFont.lfFaceName,fDEFAULT_FONT);
	theFontList->Add(new ListOfFonts(theTextFont));

	CurrentFont = 0;


	// Create a the default line thickness
	LineStyle l;

	// The default line style
	l.Colour = cBLACK;
	l.Thickness = 1;
	l.Style = PS_SOLID;
	theStyleList = new ListOfStyles(l);

	// The line style for a dashed line
	l.Colour = cBLACK;
	l.Thickness = 1;
	l.Style = PS_DASH;
	theStyleList->Add(new ListOfStyles(l));

	// The line style for a wire
	l.Colour = cWIRE;
	l.Thickness = 1;
	l.Style = PS_SOLID;
	theStyleList->Add(new ListOfStyles(l));

	// The line style for a bus
	l.Colour = cBUS;
	l.Thickness = 5;
	l.Style = PS_SOLID;
	theStyleList->Add(new ListOfStyles(l));

	CurrentStyle = 0;


	// Attempt to load the defaults for the grid
	WORD hold;
	Registry theRegistry;

	GridShow  = theRegistry.Get("ShowGrid",&hold) ? hold : FALSE;
	Units     = theRegistry.Get("Units",&hold) ? hold : 0;
	ScrollBar = theRegistry.Get("ScrollBar",&hold) ? hold : 20;

	PinLength = theRegistry.Get("PinLength", &hold) ? hold : 30;

	theMetaFileList = new ListOfMetaFiles( NULL );


	// Create the no symbol object
	CDrawSquare *Sq = new CDrawSquare(pDesign);
	Sq->a=CPoint(0,0);
	Sq->b=CPoint(-40,-40);
	Sq->next=NULL;
	CPoint tr = CPoint(-40,-40);
	CDesignFileSymbol *NoSymbol = new CDesignFileSymbol( pDesign, Sq, tr );
	NoSymbol->name = "No Symbol Defined";
	NoSymbol->reference="?";
	NoSymbol->ppp = 1;  
	theSymbolList = new ListOfSymbols( NoSymbol );
}

// The destructor
Option::~Option()
{
	delete theFontList;
	delete theStyleList;
	delete theMetaFileList;
	delete theSymbolList;
}

void Option::UnTag()
{
	theFontList->UnTag();
	theStyleList->UnTag();
	theMetaFileList->UnTag();
	theSymbolList->UnTag();
}



////// Some default settings for dialogs ////// 

void Option::SetPinLength( int pl )
{
	PinLength = pl;
	Registry theRegistry;
	theRegistry.Place("PinLength", PinLength );
}



////// Save the current settings along with the design //////

void Option::SaveOptions( CArchive &theArchive )
{
	theArchive << (BYTE)xOptions;
	theArchive << CurrentFont << CurrentStyle << GridShow << Units << ScrollBar;
}

void Option::LoadOptions( CArchive &theArchive, BOOL keep )
{
	if (keep)
	{
		theArchive >> CurrentFont >> CurrentStyle >> GridShow >> Units >> ScrollBar;
		CurrentFont = GetNewFontNumber( CurrentFont );
		CurrentStyle = GetNewStyleNumber( CurrentStyle );
	}
	else
	{
		hFONT dummy1;
		BYTE dummy2;
		LONG dummy3;
		theArchive >> dummy1 >> dummy1 >> dummy2 >> dummy3 >> dummy3;
	}
}




////// The edit options //////

// Change the size of a font and create a new entry

hFONT Option::ChangeFontSize(hFONT n, int y)
{
	LOGFONT Font = *GetFont( n );
	Font.lfHeight += y;
	return AddFont( &Font );
}





void Option::ChooseFont(CDC &PrinterDC, HWND Window)
{
	CHOOSEFONT cf;
	LOGFONT aLogFont = *(GetFont(GetCurrentFont()));

	cf.lStructSize = sizeof(CHOOSEFONT);
    cf.hwndOwner = Window;
    cf.hDC = PrinterDC.m_hDC;
    cf.lpLogFont = &aLogFont;
    cf.iPointSize = 10;
    cf.Flags = CF_FORCEFONTEXIST | CF_INITTOLOGFONTSTRUCT | CF_BOTH;
    cf.rgbColors = RGB(0,0,0);
    cf.lCustData = 0;
    cf.lpfnHook = NULL;
    cf.lpTemplateName = NULL;
    cf.hInstance = NULL;
    cf.lpszStyle = NULL;
    cf.nFontType = SCREEN_FONTTYPE;
    cf.nSizeMin = 5;
    cf.nSizeMax = 128;

	if (::ChooseFont(&cf))
		CurrentFont = AddFont(&aLogFont);
}










// The dialog box for line thickness

class CDlgLineThickBox : public CDialog {

	LineStyle theLineStyle;
	ColourSelect *theColourSelect;
	CPoint Location;

public:
	CDlgLineThickBox(CWnd *pParentWnd) : CDialog( IDD_LINETHICK, pParentWnd ) {  }

	LineStyle	*GetLineStyle() { return &theLineStyle; }
	void 		SetLineStyle(LineStyle nl) { theLineStyle = nl; }

	virtual BOOL OnInitDialog();
	virtual void OnOK();
	afx_msg void OnPaint();
	afx_msg void OnLButtonDown(UINT,CPoint);
	afx_msg void OnMeasureItem(int,LPMEASUREITEMSTRUCT);
	afx_msg void OnDrawItem( int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct );

	DECLARE_MESSAGE_MAP();

};



BEGIN_MESSAGE_MAP( CDlgLineThickBox, CDialog )
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MEASUREITEM()
	ON_WM_DRAWITEM()
END_MESSAGE_MAP()

void CDlgLineThickBox::OnLButtonDown(UINT,CPoint p)
{
	p = CPoint(p.x - Location.x,p.y -  Location.y);
	if (theColourSelect->Select(p)) {
		CClientDC dc(this);
		theColourSelect->OnPaint(dc);
	}
}


const int PenStyles[] = { PS_SOLID, PS_DASH, PS_DOT, PS_DASHDOT, PS_DASHDOTDOT };


void CDlgLineThickBox::OnMeasureItem(int,LPMEASUREITEMSTRUCT l)
{
	l->itemHeight = 12;
}

void CDlgLineThickBox::OnDrawItem( int nIDCtl, LPDRAWITEMSTRUCT l )
{

	int which = l->itemID;

	CDC *dc;
	dc = CDC::FromHandle(l->hDC);

	if (which != -1) {
	    COLORREF clrBackground = GetSysColor(l->itemState & ODS_SELECTED ? COLOR_HIGHLIGHT : COLOR_WINDOW);
		COLORREF oldBackground = dc->SetBkColor(clrBackground);

		// Draw the background box
		CBrush theBrush(clrBackground);
		CBrush *oldBrush = dc->SelectObject(&theBrush);
		CPen *oldPen = (CPen *)(dc->SelectStockObject(NULL_PEN));
		dc->Rectangle(&(l->rcItem));

		CPen thePen(PenStyles[which],1,l->itemState & ODS_SELECTED ? RGB(255,255,255) : RGB(0,0,0));
		dc->SelectObject(&thePen);
		int y = l->rcItem.top + (l->rcItem.bottom - l->rcItem.top)/2;
		int width = (l->rcItem.right - l->rcItem.left)-6;

    	dc->MoveTo(3,y);
		dc->LineTo(width,y);

		// Restore the dc to it's old state
		dc->SelectObject(oldPen);
		dc->SelectObject(oldBrush);
		dc->SetBkColor(oldBackground);
	}


   // If the item has the focus, draw focus rectangle
   if (l->itemState & ODS_FOCUS)
		DrawFocusRect(l->hDC, &l->rcItem);

}



void CDlgLineThickBox::OnPaint()
{
	CPaintDC dc(this);

	// Draw the colour selection dialog
	theColourSelect->OnPaint(dc);

	CDialog::OnPaint();
}

BOOL CDlgLineThickBox::OnInitDialog()
{
	SetDlgItemInt( LINETHICKBOX_THICKNESS, theLineStyle.Thickness );

	char hold[8];
	CComboBox *theComboBox = (CComboBox *)GetDlgItem(LINETHICKBOX_STYLE);
	hold[1] = 0;
	hold[0] = 'a';
	int which;
	for (int lp = 0; lp != 5; lp++) {
		int dwIndex = theComboBox->AddString(hold);
		theComboBox->SetItemData(dwIndex,lp);
		if (PenStyles[lp] == theLineStyle.Style)
			which = lp;
		hold[0] ++;
	}
	// Now select the correct item
	theComboBox->SetCurSel(which);

	// Create a new colour selection control
	CStatic *Static = (CStatic *)GetDlgItem(LINETHICKBOX_COLOUR);
	CRect theRect;
	Static->GetWindowRect(theRect);
	Location = CPoint(theRect.left,theRect.top);
	GetWindowRect(theRect);
	Location = CPoint(Location.x - theRect.left, Location.y - theRect.top);

	CClientDC dc(this);
	theColourSelect = new ColourSelect(dc,Location);
	theColourSelect->SetColour(theLineStyle.Colour);

	
	return TRUE;
}


void CDlgLineThickBox::OnOK()
{
	CComboBox *theComboBox = (CComboBox *)GetDlgItem(LINETHICKBOX_STYLE);

	theLineStyle.Style = PenStyles[theComboBox->GetCurSel()];
	theLineStyle.Thickness = GetDlgItemInt( LINETHICKBOX_THICKNESS );
	theLineStyle.Colour = theColourSelect->GetColour();

	EndDialog( IDOK );
}


void Option::ChooseLineStyle(CWnd *Window)
{
 	CDlgLineThickBox theDialog(Window);

	theDialog.SetLineStyle( *(GetStyle(GetCurrentStyle())) );

	if (theDialog.DoModal() != IDOK)
		return;

	CurrentStyle = AddStyle(theDialog.GetLineStyle());
}



////// The symbol operators //////

void Option::ResetSymbols()
{
	ListOfSymbols *pSymbol = theSymbolList;

	while (pSymbol != NULL)
	{
		pSymbol->ResetSymbol();
		pSymbol = static_cast<ListOfSymbols*>(pSymbol->next);
	}
}



////// The grid options //////


// The dialog to deal with the grid setup box

// The dialog for getting the grid setup details
class CDlgGridBox : public CDialog {
	LONG	Grid;
	BOOL	ShowGrid;
	BOOL	SnapGrid;
	int		Units;
	int		ScrollBar;

public:
	CDlgGridBox(CWnd *pParentWnd) : CDialog( IDD_GRID, pParentWnd ) { }

	void SetGrid(int a)       { Grid = a; }
	void SetShowGrid(BOOL a)  { ShowGrid = a; }
	void SetSnapGrid(BOOL a)  { SnapGrid = a; }
	void SetUnits(int a)      { Units = a; }
	void SetScrollBar(int a)  { ScrollBar = a; }

	LONG  GetGrid()      { return Grid; }
	BOOL GetShowGrid()  { return ShowGrid; }
	BOOL GetSnapGrid()  { return SnapGrid; }
	int  GetUnits()     { return Units; }
	int	 GetScrollBar() { return ScrollBar; }

	virtual void OnOK();
	virtual BOOL OnInitDialog();
	void OnChange();

	DECLARE_MESSAGE_MAP();
};

BEGIN_MESSAGE_MAP( CDlgGridBox, CDialog )
	ON_BN_CLICKED(GRIDBOX_MM, OnChange)
	ON_BN_CLICKED(GRIDBOX_INCHES, OnChange)
END_MESSAGE_MAP()


void CDlgGridBox::OnOK()
{
	ShowGrid  = IsDlgButtonChecked(GRIDBOX_SHOW);
	SnapGrid  = IsDlgButtonChecked(GRIDBOX_SNAP);

	int hold = GetDlgItemInt(GRIDBOX_SCROLLBAR);
	if (hold > 0)
		ScrollBar = hold;

	hold = GetCheckedRadioButton(GRIDBOX_MM,GRIDBOX_INCHES);
	if (hold == GRIDBOX_MM)
		Units = 0;
	else
		Units = 1;


	hold = GetCheckedRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER);
	switch (hold) {
		case GRIDBOX_NORMAL:
			Grid = NormalGrid;
			break;
		case GRIDBOX_FINE:
			Grid = FineGrid;
			break;
		case GRIDBOX_USER:
			Grid = GetDlgItemInt(GRIDBOX_SPACING);
			if (Units == 0)
				Grid = (Grid * PIXELSPERMM);
			else
				Grid = (Grid * 254 * PIXELSPERMM) / 1000;
			break;
	}

	if (Grid <=0 )
		Grid = FineGrid;

	EndDialog( IDOK );
}

BOOL CDlgGridBox::OnInitDialog()
{
	CheckDlgButton(GRIDBOX_SHOW,ShowGrid);
	CheckDlgButton(GRIDBOX_SNAP,SnapGrid);

	SetDlgItemInt(GRIDBOX_SCROLLBAR,ScrollBar);

	CheckRadioButton(GRIDBOX_MM,GRIDBOX_INCHES,GRIDBOX_MM+Units);
	if (Grid == NormalGrid)
		CheckRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER,GRIDBOX_NORMAL);
	else if (Grid == FineGrid)
		CheckRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER,GRIDBOX_FINE);
	else
		CheckRadioButton(GRIDBOX_NORMAL,GRIDBOX_USER,GRIDBOX_USER);

	// Set up the display of the grid spacing correctly
	OnChange();	

	return TRUE;
}

void CDlgGridBox::OnChange()
{
	// The current grid in .1mm's
	LONG ScaledGrid;

	if (GetCheckedRadioButton(GRIDBOX_MM,GRIDBOX_INCHES) == GRIDBOX_MM) {
		ScaledGrid = Grid / PIXELSPERMM;
		SetDlgItemText(GRIDBOX_UNITS,".1mm");
	} else {
		SetDlgItemText(GRIDBOX_UNITS,".001\"");
		ScaledGrid = (1000*Grid) / (254 * PIXELSPERMM);
	}

	SetDlgItemInt(GRIDBOX_SPACING,ScaledGrid);
}



// Change the grid settings
void Option::SetupGrid(TransformSnap &theTransform)
{
	CDlgGridBox theDialog(AfxGetMainWnd());

	theDialog.SetShowGrid(GridShow);
	theDialog.SetGrid(theTransform.GetAccurateGrid());
	theDialog.SetSnapGrid(theTransform.GetGridSnap());
    theDialog.SetUnits(Units);
	theDialog.SetScrollBar(ScrollBar);

	if (theDialog.DoModal() != IDOK)
		return;

	theTransform.SetAccurateGrid(theDialog.GetGrid());
	GridShow = theDialog.GetShowGrid();
	theTransform.SetGridSnap(theDialog.GetSnapGrid());
	Units = theDialog.GetUnits();
	ScrollBar = theDialog.GetScrollBar();

	
	// Write the settings to the registry
	Registry theRegistry;
	WORD hold;

	hold = GridShow;
	theRegistry.Place("ShowGrid",hold);

	hold = Units;
	theRegistry.Place("Units",hold);

	hold = ScrollBar;
	theRegistry.Place("ScrollBar",hold);

	hold = theTransform.GetAccurateGrid();
	theRegistry.Place("Grid",hold);
}


// Convert a CPoint to a Unit CString
CString Option::PointToUnit(CPoint a)
{
 	CString r;

  	float Scale = GetUnits()==0 ? (float)PIXELSPERMM : ((float)PIXELSPERMM * 25.4f);
	float sx = a.x / Scale;
	float sy = a.y / Scale;
  	sprintf(r.GetBuffer(20),"%.1f,%.1f%s",sx,sy,GetUnits()==0 ? "mm" : "\"");
	r.ReleaseBuffer();

	return r;
}




////// The colour select control //////


// The constructor
ColourSelect::ColourSelect(CDC &dc,CPoint NewLocation)
{
	// Load in the bitmap
    theBitmap.LoadBitmap("COLOURMAP");
    BITMAP theBitmapSize;
    theBitmap.GetObject(sizeof(theBitmapSize),&theBitmapSize);
    theSize=CSize(theBitmapSize.bmWidth,theBitmapSize.bmHeight);

	// Set the location to draw at
	Location = NewLocation;

	// Select the bitmap into a DC
	theBitmapDC.CreateCompatibleDC(&dc);
  	oldBitmap = (CBitmap *)(theBitmapDC.SelectObject(&theBitmap));

}


// The destructor
ColourSelect::~ColourSelect()
{
	// Destroy the bitmap
	theBitmapDC.SelectObject(oldBitmap);
	theBitmap.DeleteObject();
	theBitmapDC.DeleteDC();
}


// The drawing primitive
void ColourSelect::OnPaint(CDC &dc)
{
    // Now copy the bitmap into our window
    dc.BitBlt(Location.x,Location.y,theSize.cx,theSize.cy,&theBitmapDC,0,0,SRCCOPY);
}

BOOL ColourSelect::Select(CPoint p)
{
	// Is this inside our bitmap?
	if (p.x < 0 || p.y < 0 || p.x > theSize.cx || p.y > theSize.cy)
		return FALSE;

	// Where in the bitmap is this pointing
	CPoint Offset = CPoint((p.x - 17), (p.y - 1));
	int xpos = (Offset.x % 16);
	int ypos = (Offset.y % 16);
	
	// Is it in a box?
	if (xpos < 1 || xpos > 11)
		return FALSE;
	if (ypos < 1 || ypos > 11)
		return FALSE;

	// Now get the colour from that area
	SetColour(theBitmapDC.GetPixel(p));
	
	return TRUE;
}


void ColourSelect::SetColour(COLORREF NewColour)
{
	// Change our copy of the colour
	theColour = NewColour;

	// Now copy the colour into the inital rectangle
	CBrush theBrush(theColour), *oldBrush;
	oldBrush = theBitmapDC.SelectObject(&theBrush);
	theBitmapDC.SelectStockObject(BLACK_PEN);
	theBitmapDC.Rectangle(1,1,14,30);
	theBitmapDC.SelectObject(oldBrush);
}

