/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __OPTION_H__
#define __OPTION_H__


#include "Symbol.h"

#define fDEFAULT_FONT 	"Arial"
#define fDEFAULT_STYLE 	"Regular"


#define	hFONT			WORD					// The font handle type
#define hSTYLE			WORD					// The line style type
#define hMETAFILE		WORD
#define hRESOURCE		WORD
#define hSYMBOL			WORD

class CTinyCadDoc;

class Transform;
class TransformSnap;

class LineStyle {
public:
	LONG		Style;
	LONG		Thickness;
	COLORREF	Colour;
};


////// The resource settings //////



class CDocResource {
private:
	hRESOURCE	ResourceNumber;					// The identifier for this resource
	hRESOURCE	MergeNumber;					// Used during merging of two resource tables
	BOOL		InUse;							// Used during Garbage collection

public:

	CDocResource 	*next;							// The Next font in this list

	class LayerBox;
	friend 	LayerBox;


	// The constructor
	CDocResource();

	// The virtual destructor
	virtual ~CDocResource();

	// Convert an old resource number into a new resource number
	hRESOURCE ConvertOld(hRESOURCE n);	
	// Add a new resource to this list
	hRESOURCE Add(CDocResource *,hRESOURCE = -1);
	// Get a resource from the list
	CDocResource *Get(hRESOURCE);

	// The Garbage collection facilities
	void Tag(hRESOURCE);
	void UnTag();

	void Save(CTinyCadDoc *pDesign, CArchive &);
	void Load(CTinyCadDoc *pDesign, CArchive &);

	hRESOURCE	GetResourceNumber() { return ResourceNumber; }

	// The virtual parts of the resource
	virtual void SaveItem(CTinyCadDoc *pDesign, CArchive &) = 0;
	virtual void LoadItem(CTinyCadDoc *pDesign, CArchive &, hRESOURCE) = 0;
	virtual	BOOL Compare(CDocResource *) = 0;			// Compare this resource with another resource
};



// Storage of font settings	
class ListOfFonts : public CDocResource {
private:
	LOGFONT	FontStyle;							// The Font this describes
public:
	// The constructor
	ListOfFonts(LOGFONT f) 
	{ 
		FontStyle = f; 
	}

	virtual ~ListOfFonts()
	{
	}

	// Get the data associated with this resource
	LOGFONT *GetFontStyle() { return &FontStyle; }

	// Compare this resource with another resource of the same type
	BOOL Compare(CDocResource *);
	void SaveItem(CTinyCadDoc *pDesign, CArchive &);
	void LoadItem(CTinyCadDoc *pDesign, CArchive &, hRESOURCE);
};



// Storage of line style settings	
class ListOfStyles : public CDocResource {
private:
	LineStyle	theLineStyle;						// The Line style this describes
public:
	// The constructor
	ListOfStyles(LineStyle s) 
	{ 
		theLineStyle = s; 
	}

	virtual ~ListOfStyles()
	{
	}


	// Get the data associated with this resource
	LineStyle *GetLineStyle() { return &theLineStyle; }

	// Compare this resource with another resource of the same type
	BOOL Compare(CDocResource *);
	void SaveItem(CTinyCadDoc *pDesign, CArchive &);
	void LoadItem(CTinyCadDoc *pDesign, CArchive &, hRESOURCE);
};


class ListOfMetaFiles : public CDocResource {
	HENHMETAFILE theMetaFile;
public:
	// The constructor
	ListOfMetaFiles(HENHMETAFILE s)
	{ 
		theMetaFile = s; 
	}

	virtual ~ListOfMetaFiles()
	{
	}


	// Get the data associated with this resource
	HENHMETAFILE GetMetaFile() { return theMetaFile; }

	// Compare this resource with another resource of the same type
	BOOL Compare(CDocResource *);
	void SaveItem(CTinyCadDoc *pDesign, CArchive &);
	void LoadItem(CTinyCadDoc *pDesign, CArchive &, hRESOURCE);
};

class ListOfSymbols : public CDocResource 
{
	CDesignFileSymbol	*theSymbol;
public:
	// The constructor
	ListOfSymbols(CDesignFileSymbol *s) : CDocResource() 
	{ 
		theSymbol = s; 
	}

	// The destructor
	virtual ~ListOfSymbols();

	// Get the data associated with this resource
	CDesignFileSymbol* GetSymbol() { return theSymbol; }

	// Compare this resource with another resource of the same type
	BOOL Compare(CDocResource *);
	void SaveItem(CTinyCadDoc *pDesign, CArchive &);
	void LoadItem(CTinyCadDoc *pDesign, CArchive &, hRESOURCE);

	// Reload from the library
	void ResetSymbol();
};



class Option 
{
	CTinyCadDoc *m_pDesign;

	ListOfFonts	 *theFontList;
	ListOfStyles *theStyleList;
	ListOfMetaFiles *theMetaFileList;
	ListOfSymbols *theSymbolList;

	// The current settings
	hFONT	CurrentFont;						// The default font number
	hSTYLE	CurrentStyle;						// The default line style

	// The current grid settings
	BOOL	GridShow;							// Should we show the grid?
	LONG	Units;								// The units to use
	LONG	ScrollBar;							// The scroll bar increments

	// Some default settings for dialogs
	int		PinLength;							// The default pin length

public:

	Option();									// The Constructor
	~Option();

	Init( CTinyCadDoc *pDesign );


	// Untag all of our resources
	void UnTag();

	////// The font operators ////

	// Add a font to the list and return it's new index number
	hFONT AddFont(LOGFONT *lf,hFONT MergeNumber = -1) { return theFontList->Add(new ListOfFonts(*lf), MergeNumber); }
	// Save the font table into a file
	void SaveFonts(CArchive &theFile) { theFontList->Save(m_pDesign,theFile); }
	// Load the font table from a file
	void LoadFonts(CArchive &theFile) { theFontList->Load(m_pDesign,theFile); }
	// Convert an old font number into a new font number
	hFONT GetNewFontNumber(hFONT n) { return theFontList->ConvertOld(n); }
	// Get a font from the number
	LOGFONT *GetFont(hFONT n)  { return ((ListOfFonts *)theFontList->Get(n))->GetFontStyle(); }
	hFONT ChangeFontSize(hFONT n, int y);
	// Specify this font is in use
	void TagFont(hFONT n) { theFontList->Tag(n); }
	// Bring up the user change font box
	void ChooseFont(CDC &,HWND = NULL);	
	hFONT  GetCurrentFont() { return CurrentFont; }
	void SetCurrentFont(hFONT n) { CurrentFont = n; }


	////// The Line Style operators //////

	// Add a line style to the list and return it's new index number
	hSTYLE AddStyle(LineStyle *ls,hFONT MergeNumber = -1) { return theStyleList->Add(new ListOfStyles(*ls), MergeNumber); }
	// Save the line style table into a file
	void SaveStyles(CArchive &theFile) { theStyleList->Save(m_pDesign,theFile); }
	// Load the line style table from a file
	void LoadStyles(CArchive &theFile) { theStyleList->Load(m_pDesign,theFile); }
	// Convert an old line style number into a new font number
	hSTYLE GetNewStyleNumber(hSTYLE n) { return theStyleList->ConvertOld(n); }
	// Get a line style from the number
	LineStyle *GetStyle(hSTYLE n)  { return ((ListOfStyles *)theStyleList->Get(n))->GetLineStyle(); }
	// Specify this line style is in use
	void TagStyle(hSTYLE n) { theStyleList->Tag(n); }
	hSTYLE  GetCurrentStyle() { return CurrentStyle; }
	void SetCurrentStyle(hSTYLE n) { CurrentStyle = n; }
	// The choose box for line style
	void ChooseLineStyle(CWnd * = NULL);


	////// The Meta File operators //////

	// Add a line style to the list and return it's new index number
	hMETAFILE AddMetaFile(HENHMETAFILE s,hMETAFILE MergeNumber = -1) { return theMetaFileList->Add(new ListOfMetaFiles(s), MergeNumber); }
	// Save the line style table into a file
	void SaveMetaFiles(CArchive &theFile) { theMetaFileList->Save(m_pDesign,theFile); }
	// Load the line style table from a file
	void LoadMetaFiles(CArchive &theFile) { theMetaFileList->Load(m_pDesign,theFile); }
	// Convert an old line style number into a new font number
	hMETAFILE GetNewMetaFileNumber(hMETAFILE n) { return theMetaFileList->ConvertOld(n); }
	// Get a line style from the number
	HENHMETAFILE GetMetaFile(hMETAFILE n)  { return ((ListOfMetaFiles *)theMetaFileList->Get(n))->GetMetaFile(); }
	// Specify this line style is in use
	void TagMetaFile(hSTYLE n) { theMetaFileList->Tag(n); }



	////// The Symbols operators //////

	// Add a line style to the list and return it's new index number
	hSYMBOL AddSymbol(CDesignFileSymbol *s,hSYMBOL MergeNumber = -1) { return theSymbolList->Add(new ListOfSymbols(s), MergeNumber); }
	// Save the line style table into a file
	void SaveSymbols(CArchive &theFile) { theSymbolList->Save(m_pDesign,theFile); }
	// Load the line style table from a file
	void LoadSymbols(CArchive &theFile) { theSymbolList->Load(m_pDesign,theFile); }
	// Convert an old line style number into a new font number
	hSYMBOL GetNewSymbolNumber(hSYMBOL n) { return theSymbolList->ConvertOld(n); }
	// Get a line style from the number
	CDesignFileSymbol *GetSymbol(hSYMBOL n)  { return ((ListOfSymbols *)theSymbolList->Get(n))->GetSymbol(); }
	// Specify this line style is in use
	void TagSymbol(hSYMBOL n) { theSymbolList->Tag(n); }
	// Reload the symbols from their libraries
	void ResetSymbols();



	////// Save the current settings along with the design //////
	void SaveOptions( CArchive &theArchive );
	void LoadOptions( CArchive &theArchive, BOOL keep );



	////// The grid operators //////

	BOOL ShowGrid() { return GridShow; }		// The size of the current grid
	void SetupGrid(TransformSnap &);				// Change the grid settings
	int	 GetUnits() { return Units; }
	int  GetScrollBar() { return ScrollBar; }
	CString PointToUnit(CPoint);


	////// Some default settings for dialogs ////// 

	int		GetPinLength() { return PinLength; }
	void	SetPinLength( int pl );
};


class ColourSelect {

private:
	CBitmap 	theBitmap;
	CBitmap 	*oldBitmap;
	CDC			theBitmapDC;
	CSize 		theSize;
	CPoint		Location;
	COLORREF	theColour;

public:

	ColourSelect(CDC &,CPoint);
	virtual ~ColourSelect();

	void OnPaint(CDC &);

	BOOL Select(CPoint);
	COLORREF GetColour() { return theColour; }
	void SetColour(COLORREF);
};



#endif
